@@ -1,32 +1,18 @@
-//! # Async query support
-//! This module does not export anything, as it provides additional
-//! methods on [`WMIConnection`](WMIConnection)
-//!
-//! You only have to activate the `async-query` feature flag in Cargo.toml to use them.
-//! ```toml
-//! wmi = { version = "x.y.z",  features = ["async-query"] }
-//! ```
-//!
-//!
-
-use crate::query_sink::{IWbemObjectSink, QuerySink};
-use crate::result_enumerator::IWbemClassWrapper;
-use crate::{connection::WMIConnection, utils::check_hres, WMIResult};
 use crate::{
+    connection::WMIConnection,
     query::{build_query, FilterValue},
-    BStr,
+    query_sink::{AsyncQueryResultStream, AsyncQueryResultStreamInner, QuerySink},
+    result_enumerator::IWbemClassWrapper,
+    WMIResult,
 };
-use com::production::ClassAllocation;
-use com::AbiTransferable;
 use futures::stream::{Stream, StreamExt, TryStreamExt};
 use serde::de;
-use std::{collections::HashMap, ptr};
-use winapi::um::wbemcli::WBEM_FLAG_BIDIRECTIONAL;
+use std::collections::HashMap;
+use windows::core::BSTR;
+use windows::Win32::System::Wmi::{IWbemObjectSink, WBEM_FLAG_BIDIRECTIONAL};
 
 ///
 /// ### Additional async methods
-/// **Following methods are implemented under the
-/// `async-query` feature flag.**
 ///
 impl WMIConnection {
     /// Wrapper for the [ExecQueryAsync](https://docs.microsoft.com/en-us/windows/win32/api/wbemcli/nf-wbemcli-iwbemservices-execqueryasync)
@@ -37,27 +23,33 @@ impl WMIConnection {
         &self,
         query: impl AsRef<str>,
     ) -> WMIResult<impl Stream<Item = WMIResult<IWbemClassWrapper>>> {
-        let query_language = BStr::from_str("WQL")?;
-        let query = BStr::from_str(query.as_ref())?;
+        let query_language = BSTR::from("WQL");
+        let query = BSTR::from(query.as_ref());
 
-        let (tx, rx) = async_channel::unbounded();
+        let stream = AsyncQueryResultStreamInner::new();
         // The internal RefCount has initial value = 1.
-        let p_sink: ClassAllocation<QuerySink> = QuerySink::allocate(Some(tx));
-        let p_sink_handel = IWbemObjectSink::from(&**p_sink);
+        let p_sink = QuerySink {
+            stream: stream.clone(),
+        };
+        let p_sink_handle: IWbemObjectSink = p_sink.into();
 
         unsafe {
             // As p_sink's RefCount = 1 before this call,
             // p_sink won't be dropped at the end of ExecQueryAsync
-            check_hres((*self.svc()).ExecQueryAsync(
-                query_language.as_bstr(),
-                query.as_bstr(),
-                WBEM_FLAG_BIDIRECTIONAL as i32,
-                ptr::null_mut(),
-                p_sink_handel.get_abi().as_ptr() as *mut _,
-            ))?;
+            self.svc.ExecQueryAsync(
+                &query_language,
+                &query,
+                WBEM_FLAG_BIDIRECTIONAL,
+                &self.ctx.0,
+                &p_sink_handle,
+            )?;
         }
 
-        Ok(rx)
+        Ok(AsyncQueryResultStream::new(
+            stream,
+            self.clone(),
+            p_sink_handle,
+        ))
     }
 
     /// Async version of [`raw_query`](WMIConnection#method.raw_query)
@@ -69,13 +61,13 @@ impl WMIConnection {
     /// # use wmi::*;
     /// # use std::collections::HashMap;
     /// # use futures::executor::block_on;
-    /// # fn main() -> Result<(), wmi::WMIError> {
+    /// # fn main() -> WMIResult<()> {
     /// #   block_on(exec_async_query())?;
     /// #   Ok(())
     /// # }
     /// #
     /// # async fn exec_async_query() -> WMIResult<()> {
-    /// # let con = WMIConnection::new(COMLibrary::new()?.into())?;
+    /// # let con = WMIConnection::new(COMLibrary::new()?)?;
     /// use futures::stream::TryStreamExt;
     /// let results: Vec<HashMap<String, Variant>> = con.async_raw_query("SELECT Name FROM Win32_OperatingSystem").await?;
     /// #   Ok(())
@@ -100,13 +92,13 @@ impl WMIConnection {
     /// # use wmi::*;
     /// # use std::collections::HashMap;
     /// # use futures::executor::block_on;
-    /// # fn main() -> Result<(), wmi::WMIError> {
+    /// # fn main() -> WMIResult<()> {
     /// #   block_on(exec_async_query())?;
     /// #   Ok(())
     /// # }
     /// #
     /// # async fn exec_async_query() -> WMIResult<()> {
-    /// # let con = WMIConnection::new(COMLibrary::new()?.into())?;
+    /// # let con = WMIConnection::new(COMLibrary::new()?)?;
     /// use serde::Deserialize;
     /// #[derive(Deserialize, Debug)]
     /// struct Win32_Process {
@@ -135,7 +127,7 @@ impl WMIConnection {
     where
         T: de::DeserializeOwned,
     {
-        let query_text = build_query::<T>(Some(&filters))?;
+        let query_text = build_query::<T>(Some(filters))?;
 
         self.async_raw_query(&query_text).await
     }
@@ -145,9 +137,9 @@ impl WMIConnection {
 #[allow(non_camel_case_types)]
 #[cfg(test)]
 mod tests {
-    use crate::tests::fixtures::*;
-    use crate::Variant;
-    use futures::stream::StreamExt;
+    use crate::{tests::fixtures::*, Variant};
+    use futures::stream::{self, StreamExt};
+    use serde::Deserialize;
     use std::collections::HashMap;
 
     #[async_std::test]
@@ -163,6 +155,19 @@ mod tests {
         assert_eq!(result.len(), 1);
     }
 
+    #[tokio::test]
+    async fn async_it_works_async_tokio() {
+        let wmi_con = wmi_con();
+
+        let result = wmi_con
+            .exec_query_async_native_wrapper("SELECT OSArchitecture FROM Win32_OperatingSystem")
+            .unwrap()
+            .collect::<Vec<_>>()
+            .await;
+
+        assert_eq!(result.len(), 1);
+    }
+
     #[async_std::test]
     async fn async_it_handles_invalid_query() {
         let wmi_con = wmi_con();
@@ -187,14 +192,43 @@ mod tests {
 
         for res in results {
             match res.get("GroupComponent") {
-                Some(Variant::String(s)) => assert!(s != ""),
+                Some(Variant::String(s)) => assert_ne!(s, ""),
                 _ => assert!(false),
             }
 
             match res.get("PartComponent") {
-                Some(Variant::String(s)) => assert!(s != ""),
+                Some(Variant::String(s)) => assert_ne!(s, ""),
                 _ => assert!(false),
             }
         }
     }
+
+    #[tokio::test]
+    async fn async_it_works_async_tokio_concurrent() {
+        let wmi_con = wmi_con();
+
+        // We want to actually consume a bunch of data from WMI.
+        #[allow(unused)]
+        #[derive(Deserialize, Debug)]
+        struct Win32_OperatingSystem {
+            Name: String,
+            SerialNumber: String,
+            OSArchitecture: String,
+            BootDevice: String,
+            MUILanguages: Vec<String>,
+        }
+
+        // Using buffer_unordered(1) will take 2 seconds instead of 0.2 seconds.
+        let results: Vec<Win32_OperatingSystem> = stream::iter(0..150)
+            .map(|_| async {
+                let result: Vec<Win32_OperatingSystem> = wmi_con.async_query().await.unwrap();
+
+                result.into_iter().next().unwrap()
+            })
+            .buffer_unordered(50)
+            .collect()
+            .await;
+
+        assert_eq!(results.len(), 150);
+    }
 }
