@@ -1,45 +1,42 @@
-use serde::de::{
-    self, DeserializeOwned, DeserializeSeed, EnumAccess, IntoDeserializer, MapAccess, Unexpected,
-    VariantAccess, Visitor,
+use crate::{result_enumerator::IWbemClassWrapper, WMIError, WMIResult};
+use serde::{
+    de::{
+        self, DeserializeOwned, DeserializeSeed, EnumAccess, IntoDeserializer, MapAccess,
+        Unexpected, VariantAccess, Visitor,
+    },
+    forward_to_deserialize_any,
 };
-use serde::forward_to_deserialize_any;
-
 use std::iter::Peekable;
 
-use crate::result_enumerator::IWbemClassWrapper;
-use crate::WMIError;
-
-pub struct Deserializer<'a> {
-    pub wbem_class_obj: &'a IWbemClassWrapper,
+pub struct Deserializer {
+    pub wbem_class_obj: IWbemClassWrapper,
 }
 
-impl<'a> Deserializer<'a> {
-    pub fn from_wbem_class_obj(wbem_class_obj: &'a IWbemClassWrapper) -> Self {
+impl Deserializer {
+    pub fn from_wbem_class_obj(wbem_class_obj: IWbemClassWrapper) -> Self {
         Deserializer { wbem_class_obj }
     }
 }
 
-pub fn from_wbem_class_obj<T>(wbem_class_obj: &IWbemClassWrapper) -> Result<T, WMIError>
+pub fn from_wbem_class_obj<T>(wbem_class_obj: IWbemClassWrapper) -> WMIResult<T>
 where
     T: DeserializeOwned,
 {
     let mut deserializer = Deserializer::from_wbem_class_obj(wbem_class_obj);
-    let t = T::deserialize(&mut deserializer)?;
-
-    Ok(t)
+    T::deserialize(&mut deserializer)
 }
 
-struct WMIEnum<'a, 'de: 'a> {
-    de: &'a mut Deserializer<'de>,
+struct WMIEnum<'a> {
+    de: &'a mut Deserializer,
 }
 
-impl<'a, 'de> WMIEnum<'a, 'de> {
-    pub fn new(de: &'a mut Deserializer<'de>) -> Self {
+impl<'a> WMIEnum<'a> {
+    pub fn new(de: &'a mut Deserializer) -> Self {
         Self { de }
     }
 }
 
-impl<'de, 'a> EnumAccess<'de> for WMIEnum<'a, 'de> {
+impl<'de, 'a> EnumAccess<'de> for WMIEnum<'a> {
     type Error = WMIError;
     type Variant = Self;
 
@@ -52,9 +49,15 @@ impl<'de, 'a> EnumAccess<'de> for WMIEnum<'a, 'de> {
     }
 }
 
-impl<'de, 'a> VariantAccess<'de> for WMIEnum<'a, 'de> {
+impl<'de, 'a> VariantAccess<'de> for WMIEnum<'a> {
     type Error = WMIError;
 
+    // All other possible enum variants are not supported.
+    fn unit_variant(self) -> Result<(), Self::Error> {
+        let unexp = Unexpected::UnitVariant;
+        Err(de::Error::invalid_type(unexp, &"newtype variant"))
+    }
+
     // Newtype variants can be deserialized directly.
     fn newtype_variant_seed<T>(self, seed: T) -> Result<T::Value, Self::Error>
     where
@@ -63,12 +66,6 @@ impl<'de, 'a> VariantAccess<'de> for WMIEnum<'a, 'de> {
         seed.deserialize(self.de)
     }
 
-    // All other possible enum variants are not supported.
-    fn unit_variant(self) -> Result<(), Self::Error> {
-        let unexp = Unexpected::UnitVariant;
-        Err(de::Error::invalid_type(unexp, &"newtype variant"))
-    }
-
     fn tuple_variant<V>(self, _len: usize, _visitor: V) -> Result<V::Value, Self::Error>
     where
         V: Visitor<'de>,
@@ -90,21 +87,21 @@ impl<'de, 'a> VariantAccess<'de> for WMIEnum<'a, 'de> {
     }
 }
 
-struct WMIMapAccess<'a, 'de, S, I>
+struct WMIMapAccess<'a, S, I>
 where
     S: AsRef<str>,
     I: Iterator<Item = S>,
 {
     fields: Peekable<I>,
-    de: &'a Deserializer<'de>,
+    de: &'a Deserializer,
 }
 
-impl<'a, 'de, S, I> WMIMapAccess<'a, 'de, S, I>
+impl<'a, S, I> WMIMapAccess<'a, S, I>
 where
     S: AsRef<str>,
     I: Iterator<Item = S>,
 {
-    pub fn new(fields: I, de: &'a Deserializer<'de>) -> Self {
+    pub fn new(fields: I, de: &'a Deserializer) -> Self {
         Self {
             fields: fields.peekable(),
             de,
@@ -112,7 +109,7 @@ where
     }
 }
 
-impl<'de, 'a, S, I> MapAccess<'de> for WMIMapAccess<'a, 'de, S, I>
+impl<'de, 'a, S, I> MapAccess<'de> for WMIMapAccess<'a, S, I>
 where
     S: AsRef<str>,
     I: Iterator<Item = S>,
@@ -149,7 +146,7 @@ where
     }
 }
 
-impl<'de, 'a> de::Deserializer<'de> for &'a mut Deserializer<'de> {
+impl<'de, 'a> de::Deserializer<'de> for &'a mut Deserializer {
     type Error = WMIError;
 
     fn deserialize_any<V>(self, _visitor: V) -> Result<V::Value, Self::Error>
@@ -161,27 +158,6 @@ impl<'de, 'a> de::Deserializer<'de> for &'a mut Deserializer<'de> {
         ))
     }
 
-    fn deserialize_enum<V>(
-        mut self,
-        _name: &'static str,
-        _variants: &'static [&'static str],
-        visitor: V,
-    ) -> Result<V::Value, Self::Error>
-    where
-        V: Visitor<'de>,
-    {
-        visitor.visit_enum(WMIEnum::new(&mut self))
-    }
-
-    // When deserializing enums, return the object's class name as the expected enum variant.
-    fn deserialize_identifier<V>(self, visitor: V) -> Result<V::Value, Self::Error>
-    where
-        V: Visitor<'de>,
-    {
-        let class_name = self.wbem_class_obj.class()?;
-        visitor.visit_string(class_name)
-    }
-
     // Support for deserializing `Wrapper(Win32_OperatingSystem)`.
     fn deserialize_newtype_struct<V>(
         self,
@@ -200,7 +176,7 @@ impl<'de, 'a> de::Deserializer<'de> for &'a mut Deserializer<'de> {
     {
         let fields = self.wbem_class_obj.list_properties()?;
 
-        visitor.visit_map(WMIMapAccess::new(fields.iter(), &self))
+        visitor.visit_map(WMIMapAccess::new(fields.iter(), self))
     }
 
     fn deserialize_struct<V>(
@@ -212,26 +188,57 @@ impl<'de, 'a> de::Deserializer<'de> for &'a mut Deserializer<'de> {
     where
         V: Visitor<'de>,
     {
-        visitor.visit_map(WMIMapAccess::new(fields.iter(), &self))
+        visitor.visit_map(WMIMapAccess::new(fields.iter(), self))
+    }
+
+    fn deserialize_enum<V>(
+        self,
+        _name: &'static str,
+        _variants: &'static [&'static str],
+        visitor: V,
+    ) -> Result<V::Value, Self::Error>
+    where
+        V: Visitor<'de>,
+    {
+        visitor.visit_enum(WMIEnum::new(self))
+    }
+
+    // When deserializing enums, return the object's class name as the expected enum variant.
+    fn deserialize_identifier<V>(self, visitor: V) -> Result<V::Value, Self::Error>
+    where
+        V: Visitor<'de>,
+    {
+        let class_name = self.wbem_class_obj.class()?;
+        visitor.visit_string(class_name)
+    }
+
+    fn deserialize_unit<V>(self, visitor: V) -> Result<V::Value, Self::Error>
+    where
+        V: Visitor<'de>,
+    {
+        visitor.visit_unit()
     }
 
     forward_to_deserialize_any! {
         bool i8 i16 i32 i64 u8 u16 u32 u64 f32 f64 char str string bytes
-        byte_buf option unit unit_struct seq tuple
+        byte_buf option unit_struct seq tuple
         tuple_struct ignored_any
     }
 }
 
 #[allow(non_snake_case)]
 #[allow(non_camel_case_types)]
+#[allow(dead_code)]
 #[cfg(test)]
 mod tests {
     use super::*;
 
     use crate::duration::WMIDuration;
     use crate::variant::Variant;
+    use crate::FilterValue;
     use serde::Deserialize;
     use std::collections::HashMap;
+    use std::time::Duration;
 
     use crate::tests::fixtures::*;
     use std::process;
@@ -244,7 +251,7 @@ mod tests {
         struct Win32_OperatingSystem {
             Caption: String,
             Name: String,
-            
+
             CurrentTimeZone: i16,
             Debug: bool,
 
@@ -254,7 +261,7 @@ mod tests {
 
             #[cfg(feature = "chrono")]
             LastBootUpTime: crate::WMIDateTime,
-            
+
             #[cfg(all(feature = "time", not(feature = "chrono")))]
             LastBootUpTime: crate::WMIOffsetDateTime,
         }
@@ -266,15 +273,15 @@ mod tests {
         for res in enumerator {
             let w = res.unwrap();
 
-            let w: Win32_OperatingSystem = from_wbem_class_obj(&w).unwrap();
+            let w: Win32_OperatingSystem = from_wbem_class_obj(w).unwrap();
 
             assert!(w.Caption.contains("Microsoft "));
             assert!(w.Name.contains("Microsoft ") && w.Name.contains("Partition"));
             assert_eq!(w.Debug, false);
             assert_eq!(w.EncryptionLevel, 256);
             assert_eq!(w.ForegroundApplicationBoost, 2);
-            assert_ne!(w.CurrentTimeZone, i16::max_value());
-            
+            assert_ne!(w.CurrentTimeZone, i16::MAX);
+
             #[cfg(any(feature = "time", feature = "chrono"))]
             assert!(w.LastBootUpTime.0.to_string().starts_with("20"));
         }
@@ -291,7 +298,7 @@ mod tests {
         for res in enumerator {
             let w = res.unwrap();
 
-            let w: HashMap<String, Variant> = from_wbem_class_obj(&w).unwrap();
+            let w: HashMap<String, Variant> = from_wbem_class_obj(w).unwrap();
 
             match w.get("Caption").unwrap() {
                 Variant::String(s) => assert!(s.starts_with("Microsoft Windows")),
@@ -304,7 +311,7 @@ mod tests {
 
             match langs {
                 Variant::Array(langs) => {
-                    assert!(langs.contains(&Variant::String("en-US".into())));
+                    assert!(matches!(langs[0], Variant::String(_)));
                 }
                 _ => assert!(false),
             }
@@ -322,7 +329,7 @@ mod tests {
         for res in enumerator {
             let w = res.unwrap();
 
-            let w: HashMap<String, Variant> = from_wbem_class_obj(&w).unwrap();
+            let w: HashMap<String, Variant> = from_wbem_class_obj(w).unwrap();
 
             match w.get("Caption").unwrap() {
                 Variant::String(s) => assert!(s.starts_with("Microsoft Windows")),
@@ -438,35 +445,99 @@ mod tests {
     }
 
     #[test]
-    fn it_can_desr_newtype_enum() {
+    fn it_can_desr_newtype_enum_field() {
         let wmi_con = wmi_con();
 
         #[derive(Deserialize, Debug)]
-        pub struct Win32_UserAccount {
-            pub __Path: String,
-            pub Name: String,
+        struct Win32_Process {}
+
+        #[derive(Deserialize, Debug)]
+        struct Win32_Thread {}
+
+        #[derive(Deserialize, Debug)]
+        enum Instance {
+            #[serde(rename = "Win32_Process")]
+            Process(Win32_Process),
+            #[serde(rename = "Win32_Thread")]
+            Thread(Win32_Thread),
         }
 
         #[derive(Deserialize, Debug)]
-        pub struct Win32_SystemAccount {
-            pub Name: String,
+        struct __InstanceCreationEvent {
+            TargetInstance: Instance,
+        }
+
+        let mut filters_process = HashMap::new();
+
+        filters_process.insert(
+            "TargetInstance".to_owned(),
+            FilterValue::is_a::<Win32_Process>().unwrap(),
+        );
+
+        filters_process.insert(
+            "TargetInstance.Name".to_owned(),
+            FilterValue::String("ping.exe".to_owned()),
+        );
+
+        let mut instances_iter = wmi_con
+            .filtered_notification::<__InstanceCreationEvent>(
+                &filters_process,
+                Some(Duration::from_secs(1)),
+            )
+            .unwrap();
+
+        std::process::Command::new("ping.exe")
+            .arg("127.0.0.1")
+            .status()
+            .unwrap();
+
+        let proc = instances_iter.next().unwrap().unwrap();
+
+        assert!(matches!(proc.TargetInstance, Instance::Process(..)))
+    }
+
+    #[test]
+    fn it_can_desr_unit_enum_field_from_string() {
+        let wmi_con = wmi_con();
+
+        #[derive(Deserialize, Debug, PartialEq, Eq)]
+        enum Status {
+            SomeValue,
+            OK,
+            SomeOtherValue,
         }
 
         #[derive(Deserialize, Debug)]
-        enum User {
-            #[serde(rename = "Win32_SystemAccount")]
-            System(Win32_SystemAccount),
-            #[serde(rename = "Win32_UserAccount")]
-            User(Win32_UserAccount),
+        struct Win32_OperatingSystem {
+            Status: Status,
         }
 
-        let user: Win32_UserAccount = wmi_con.get().unwrap();
+        let os: Win32_OperatingSystem = wmi_con.get().unwrap();
 
-        let user_enum: User = wmi_con.get_by_path(&user.__Path).unwrap();
+        assert_eq!(os.Status, Status::OK);
+    }
 
-        match user_enum {
-            User::System(_) => assert!(false),
-            User::User(_) => assert!(true),
-        };
+    #[test]
+    fn it_fail_to_desr_unit_enum_field_from_unexpected_string() {
+        let wmi_con = wmi_con();
+
+        #[derive(Deserialize, Debug, PartialEq, Eq)]
+        enum Status {
+            Potato,
+        }
+
+        #[derive(Deserialize, Debug)]
+        struct Win32_OperatingSystem {
+            Status: Status,
+        }
+
+        let res: Result<Win32_OperatingSystem, WMIError> = wmi_con.get();
+
+        let err = res.err().unwrap();
+
+        assert_eq!(
+            format!("{}", err),
+            "unknown variant `OK`, expected `Potato`"
+        )
     }
 }
